#!/bin/sh
# Deploy langpack-o-matic charm
# Author: Martin Pitt <martin.pitt@ubuntu.com>

set -e

CHARM_RELEASE=xenial

# settings for deployment in private Canonistack or juju-local

# settings for Canonical ProdStack 4.5
if [ "$OS_USERNAME" = "prod-ues-langpack" ]; then
    LANDSCAPE=1
fi

CHARMDIR=$(dirname $(readlink -f $0))

#
# Juju 1 vs. 2 abstractions
#

if juju --version 2>&1 | grep -q '^2'; then
    JUJU=juju
    unset JUJU1
elif type juju-1 >/dev/null 2>&1; then
    JUJU=juju-1
    JUJU1=1
else
    JUJU=juju
    JUJU1=1
fi

if [ -n "$JUJU1" ]; then
    JUJU_SET="$JUJU set"
else
    JUJU_SET="$JUJU set-config"
fi

deploy_charm() {
    local charm=$1
    shift
    if [ -n "$JUJU1" ]; then
        $JUJU deploy --repository "$CHARMDIR" "$@" local:$CHARM_RELEASE/$charm $APPNAME
    else
        $JUJU deploy --series $CHARM_RELEASE "$@" "$CHARMDIR/$CHARM_RELEASE/$charm" $APPNAME
    fi
}

#
# helper functions
#

wait_deployed() {
    echo "waiting for $1 to get deployed..."
    while ! $JUJU status --format=short $1 | egrep 'active|started|idle'; do
        sleep 5
    done
}

subordinate_charms() {
    if [ -n "$LANDSCAPE" ]; then
        echo 'deploying subordinate charms to $1'
        $JUJU add-relation ksplice "$1"
        $JUJU add-relation landscape-client "$1"
    fi
}

CREDENTIALS_DIR="$1"

if [ -z "$1" ] || [ -n "$2" ]; then
   echo "Usage: $0 <credentials dir>" >&2
   exit 1
fi

if [ ! -d "$CREDENTIALS_DIR" ]; then
   echo "ERROR: $CREDENTIALS_DIR does not exist" >&2
   exit 1
fi

# create charm config

config_yaml=$(mktemp)
trap "rm $config_yaml" EXIT INT QUIT PIPE

#
# deploy subordinate charms for automatic machine management (only if
# $LANDSCAPE is set, i. e. for the production environment in ProdStack)
#

if [ -n "$LANDSCAPE" ]; then
    [ -d "$CHARMDIR/$CHARM_RELEASE/landscape-client" ] || bzr checkout --lightweight lp:charms/trusty/landscape-client "$CHARMDIR/$CHARM_RELEASE/landscape-client"
    [ -d "$CHARMDIR/$CHARM_RELEASE/ksplice" ] || { echo "Please check out ksplice charm to $CHARMDIR/$CHARM_RELEASE"; exit 1; }

    #
    # install/update basenode into charms
    #
    [ -d "$CHARMDIR/basenode" ] || { echo "Please check out basenode into $CHARMDIR"; exit 1; }
    for charmdir in $CHARMDIR/$CHARM_RELEASE/*; do
        # ignore subordinate charms
        if grep -q 'subordinate:.*true' $charmdir/metadata.yaml; then
            continue
        fi
        echo "Installing basenode into $charmdir"
        rm -rf "$charmdir/exec.d/basenode"
        mkdir -p "$charmdir/exec.d"
        cp -r "$CHARMDIR/basenode" "$charmdir/exec.d"
    done

    if ! $JUJU status | grep -q ksplice; then
        deploy_charm ksplice
        $JUJU_SET ksplice accesskey=$(cat /srv/mojo/LOCAL/${MOJO_PROJECT}/canonical-is-ksplice.key)
        $JUJU_SET ksplice source=""
    fi
    if ! $JUJU status | grep -q landscape-client; then
        cat <<EOF >> "$config_yaml"
landscape-client:
  url: https://landscape.is.canonical.com/message-system
  ping-url: http://landscape.is.canonical.com/ping
  account-name: standalone
  registration-key: $(cat /srv/mojo/LOCAL/${MOJO_PROJECT}/canonical-is-landscape.key)
  tags: juju-managed, devops-instance, devops-production
EOF
        deploy_charm landscape-client --config $config_yaml
    fi
fi

#
# deploy langpack-o-matic  charm
#

if $JUJU status | grep -q langpack-o-matic; then
    echo 'WARNING: langpack-o-matic already deployed, skipping' >&2
else
    cat <<EOF >> "$config_yaml"
langpack-o-matic:
  gpg-public: |
$(cat "$CREDENTIALS_DIR/langpack.asc" | sed 's/^/    /')
  gpg-private: |
$(cat "$CREDENTIALS_DIR/langpack.priv.asc" | sed 's/^/    /')
  lp-credentials: |
$(cat "$CREDENTIALS_DIR/launchpad-export-settings.credentials" | sed 's/^/    /')
EOF

    deploy_charm langpack-o-matic --config "$config_yaml" --constraints 'root-disk=40G'
    wait_deployed langpack-o-matic
    subordinate_charms langpack-o-matic
fi
